#include "TMCgen1.h"

#include <stdio.h>

////////////////////////////////////////////////////////////////////////////////
///     TMCgen1 class
/// This is class for axiliary exercises, 
/// mainly checking analytical integration with Monte Carlo

ClassImp(TMCgen1);

TMCgen1::TMCgen1():
  TMCgen()
{
  /// This constructor is for ROOT streamers ONLY
  cout<< "----> TMCgen1 Default Constructor (for ROOT only) "<<endl;
  //m_WTmonitor == NULL;
  //m_PlotSelector = " ";
  //m_MEtype  = "";
  //m_Xnorm   = 1;       
cout << "----> TMCgen1 Default Constructor (for ROOT only): " << m_MEtype;
}

///_____________________________________________________________
TMCgen1::TMCgen1(const char* Name):
  TMCgen(Name)
{
//! Constructor to be used by the user!!!
//! Its important role is to define ALL DEFAULTS.
//! to changed by the user before calling TMCgen::Initialize
  cout<< "----> TMCgen1::TMCgen1 USER Constructor "<<endl;
  m_pi = 3.14159;
  m_MEtype  = "";
  //m_PlotSelector = "blank";
  m_Xnorm   = 1;           //! normalization
}///

///______________________________________________________________________________________
TMCgen1::~TMCgen1()
{
  //!Explicit destructor
  cout<< "----> TMCgen1::TMCgen1 !!!! DESTRUCTOR !!!! "<<endl;
}///destructor

///______________________________________________________________________________________
void TMCgen1::Initialize(TRandom *RNgen, ofstream *OutFile, TH1D* h_NORMA)
{
  /// The important and subtle mechanism of preventing repetition
  /// of the initializatin is implemented here and in the base class
  /// Initialize of the base class restores certain pointers (like redresss)
  cout<< "----> TMCgen1::Initialize, Entering "<<endl;
  TMCgen::Initialize(  RNgen, OutFile, h_NORMA);
  ///
  if(f_IsInitialized == 0)
  {
  ///=================================================================
  ///         SETTING UP FOAM
  int nDim = 1;
  if ((m_MEtype == "Example") || (m_MEtype == "Camel")) {
    nDim=2;
  }else{
    cout << "TMCgen1::Initialize() - wrong MEtype: " << m_MEtype << endl;
  }

  ///
  f_FoamI   = new TFoam("FoamI");
  f_FoamI->SetkDim(nDim);      //! No. of dimensions, obligatory!
  //f_FoamI->SetnCells(10000);    //! No. of cells, can be omitted, default=2000
  //f_FoamI->SetnSampl(100000);    //! optional
  //f_FoamI->SetOptRej(0);       //! wted events optional
  ///------------------------------------------------
  //m_FoamMode=0;                            /// initialization for abs(dist)
  f_FoamI->Initialize( f_RNgen, this);     /// Initialize Foam
  m_FoamMode=1;                            /// generation dist without abs
  double errel;
  //f_FoamI->GetIntNorm( m_Xnorm,errel);     /// universal normalization
  
  ///[[[[[[[[[[
  //m_WTmonitor = new THwtMon("WTmonitor");
  ///]]]]]]]]]]

  BXOPE(*f_Out);
  BXTXT(*f_Out,"========================================");
  BXTXT(*f_Out,"======   TMCgen1::Initialize     ======");
  BXTXT(*f_Out,"========================================");
  BX1F(*f_Out,"   Xnorm ", m_Xnorm,  "overall MC normalization   =");
  BXTXT(*f_Out,"========================================");

  f_IsInitialized = 1;  /// re-initilalization inhibited
}else{
  /// additional redressing, if necessary
  cout<< "----> TMCgen1::Initialize, already initialized "<<endl;
  Redress(RNgen, OutFile, h_NORMA);
}/// if(isInitilaized)
}/// Initialize

//usunac
///________________________________________________________________________
void TMCgen1::Redress(TRandom *RNgen, ofstream *OutFile, TH1D* h_NORMA)
{
    //f_FoamI->ResetRho(this);    /// seems to be not necessary???
  f_FoamI->ResetPseRan(RNgen);  /// restore random number generator
  cout<<" TMCgen1::Redress: )))))))"<<endl;
}///


void TMCgen1::Generate()
{
  ///------------------------------------------------
  f_NevGen++;
  ///  Fill in special normalization histogram
  f_TMCgen_NORMA->Fill(0.5, m_Xnorm);   /// 1-st bin = Normal*Nevtot
  f_TMCgen_NORMA->Fill(1.5, 1);         /// 2-nd bin = Nevtot
  ///
  //m_FoamMode=1;
  f_FoamI->MakeEvent();         // generate k and x
  m_WT   = f_FoamI->GetMCwt();  // get weight
  //	m_WTmonitor->Fill(m_WT);
}//! Generate


///______________________________________________________________________________________
void TMCgen1::Finalize()
{
  TMCgen::Finalize();
  ///===================================================
  ///   Finalize MC  run, final printouts, cleaning etc.
  BXOPE(*f_Out);
  BXTXT(*f_Out,"****************************************");
  BXTXT(*f_Out,"******     TMCgen1::Finalize     ******");
  BXTXT(*f_Out,"****************************************");
  ///------------------------
  Double_t MCresult, MCerror;
  f_FoamI->GetIntegMC( MCresult, MCerror);  //! get MC integral, should be one
  cout << "**************************************************************"<<endl;
  cout << "**************** MCgenFoam::Finalize  ************************"<<endl;
  cout << "Directly from FOAM: MCresult= " << MCresult << " +- "<<MCerror <<endl;
  cout << "**************************************************************"<<endl;
}//!Finalize

///________________________________________________________________________
double TMCgen1::Density(int nDim, double *Xarg){
  double dist;
  ///
  dist  = 1.0;
 if(m_MEtype == "Example"){
    dist = Example(nDim, Xarg);	
  }else if(m_MEtype == "Camel"){ 
    dist = Camel(nDim, Xarg);
/*  }else if(m_MEtype == "Bx"){ 
    dist = Bx(nDim, Xarg);*/
  }else{
    StopM("TMCgen1::Density: +++ wrong m_MEtype");
  }
  if(m_FoamMode == 0) dist=fabs(dist);
  return dist;
}/// Density

double TMCgen1::Example(int nDim, Double_t *Xarg)
{
  m_x = Xarg[0];
  m_y = Xarg[1];
  double dist = 1e-20;
  double r = 0.45;
  double z2 = sqr(m_x-0.5) + sqr(m_y-0.5);
  if (z2 <= sqr(r))// (sqr(m_x-0.5)< 1./16 && sqr(m_y-0.5) < 1./16)//
		dist = sqrt(r*r - z2)/(2./3 *3.14159*r*r*r);
  return dist;
}//Example

double TMCgen1::Camel(int nDim, Double_t *Xarg)
{
  m_x = Xarg[0];
  m_y = Xarg[1];
  double GamSq= sqr(0.100);
  double Dist= 0;
  Dist +=exp(-(sqr(m_x-1./3) +sqr(m_y-1./3))/GamSq)/GamSq/m_pi;
  Dist +=exp(-(sqr(m_x-2./3) +sqr(m_y-2./3))/GamSq)/GamSq/m_pi;
  return 0.5*Dist;
}//Camel
/*
double TMCgen1::Bx(double kT1, double kT2, double alf1, double alf2, double phi, double delta)
{
/// Double Bremstrahlung interference Xsed diagram
    double xx  =1-alf1-alf2;
    Double_t T1x,T2xa,T2xb,T3x,q2,rhoX;
    q2 = (1-alf2)/alf1*sqr(kT1) +(1-alf1)/alf2*sqr(kT2)+ 2*kT1*kT2*cos(phi);
    T1x=  xx*(1+sqr(xx)-alf1*alf2);
    T2xa= -(1+sqr(xx))*(1-alf2) -xx*(1-alf1);
    T2xb= -(1+sqr(xx))*(1-alf1) -xx*(1-alf2);
    T3x=  2*(1+sqr(xx));
    rhoX = 2*T1x/(alf1*alf2) 
           -T2xa* kT1*kT2*cos(phi)/sqr(kT2)/alf1
           -T2xb* kT1*kT2*cos(phi)/sqr(kT1)/alf2
           +T3x*sqr(kT1*kT2*cos(phi))/sqr(kT1*kT2);
    rhoX *= sqr(kT1)*sqr(kT2)/sqr(q2);
    rhoX *= 1/sqr(kT1*kT2);
    rhoX *= alf1/(sqr(delta)+sqr(alf1)) *alf2/(sqr(delta)+sqr(alf2));
    return rhoX;
}//Bx
*/
//===========================================================================


