// ROOT headers
#include <TROOT.h>
#include <TFile.h>
#include <TApplication.h>
#include <TCanvas.h>
#include <TLatex.h>
#include "TH1D.h"
#include "TH2D.h"
#include "TMath.h"
#include "TString.h"
// own headers
//#include "TMCgen.h"
#include "TMCgen1.h"
#include "Kfig1.h"

TFile MCgenFile("./work/mcgen.root"); //! archive root file
TFile DiskFileA("./work/histo.root"); //! archive root file

Kfig1 *KFIGobj = new Kfig1();


int DrawPlot()
{
  DiskFileA.ls();
  cout<<" ========================= DrawPlot =========================== "<<endl;
//!***********************************************************************
  TH2D *H2 = (TH2D*)DiskFileA.Get("DistributionHisto");
  Float_t  WidPix, HeiPix;
  WidPix = 500; HeiPix =  400;
  TCanvas *cCyalfr = new TCanvas("Plot","Plot", 100,100, WidPix,HeiPix);
  cCyalfr->SetFillColor(10);
  cCyalfr->Draw();
  TH2D *Hst;
  TString SurfTyp="surf2";  //! smooth coloured
  //!-----------------------------
  cCyalfr->cd();
  
  Hst =H2;
  Hst->GetXaxis()->SetNdivisions(5);
  Hst->GetXaxis()->SetTitleOffset(2.0);
  Hst->GetXaxis()->CenterTitle();
  Hst->GetXaxis()->SetTitle("x");
  //
  Hst->GetYaxis()->SetTitleOffset(2.0);
  Hst->GetYaxis()->SetNdivisions(3);
  Hst->GetYaxis()->CenterTitle();
  Hst->GetYaxis()->SetTitle("y");
  Hst->DrawCopy(SurfTyp);
  
  cCyalfr->Update();
  cCyalfr->cd();
  cout<< " **************** DrawPlot ends *******************   "<<endl;
  //
  return 0;
}
// Draws plots of the Monte Carlo distribution together with 
// the corresponding analytical formula
// if argument eqals 0, the xscale is linear
// if 1, logarithmic
int Draw_dummy(int klog)
//function for testing TKfig and Kfig1
{
 cout<< " **************** Draw_dummy starts *******************   "<<endl;
   TH1D* H_x;
   TString kaxis;
   TString cname = KFIGobj->m_PlotSelector;
   
   //TString type = "I"; //"ONE";
   TString type = KFIGobj->m_PlotSelector;
   cout << endl << "plot type: " << type << endl;
   if(klog == 0){
     TH2D *H2 = (TH2D*)DiskFileA.Get("DistributionHisto");
     //cout << "cos poszlo";	
     H_x   = (TH1D*)DiskFileA.Get("h_x");  kaxis = "lin_x";
     //TH1D* H_x   = (TH1D*)DiskFileA.Get("h_test");  
     kaxis = "lin_x";
     cout << " " << kaxis;
   }else if (klog == 1){
     H_x   = (TH1D*)DiskFileA.Get("h_lx"); kaxis = "log_x";
     //H_x   = (TH1D*)DiskFileA.Get("histogram testowy");  
     kaxis = "log_x";
     cout << " " << kaxis;
   } else { cout<<"++++ wrong klog"; exit(9);}
   if (!H_x) cout << endl << "@@@ no H_x !!! ";
   TH1D *HA_x   =  KFIGobj->DefHist1D(kaxis,type,H_x,"HA_x", "x distribution"); 

   if (!HA_x) cout << endl << "@@@ problem with  HA_x !!! ";
   HA_x->SetLineColor(kRed); ///analytic
   H_x->SetLineColor(kBlue); ///MC

  Float_t  WidPix, HeiPix;
  WidPix = 800; HeiPix =  400;
  TCanvas *cDisxNLL = new TCanvas(cname+kaxis, cname+kaxis,60,60, WidPix,HeiPix);

  H_x->GetXaxis()->SetTitle("x");
  if (klog == 1) H_x->GetXaxis()->SetTitle("log10(x)");
  H_x->DrawCopy("h");
  HA_x->DrawCopy("hsame");
cout<< " **************** Draw_dummy ends *******************   "<<endl;	
}
//////////////////////////////////////////////////////////////////////////////////
int main(int argc, char **argv)
{
  /// Normalizing all TH1D and TH2D histograms 
  KFIGobj->HistNormalize("h_TMCgen_NORMA", &DiskFileA);
//!***********************************************************************
/// Transfering parameters from MC generator to Kfig object
  MCgenFile.ls();
  TMCgen1 *MCgenerator = (TMCgen1*)MCgenFile.Get("MCgen");
  KFIGobj->m_PlotSelector   =((TMCgen1*)MCgenerator)->m_MEtype;
  KFIGobj->Info();
  ///++++++++++++++++++++++++++++++++++++++++
  TApplication theApp("theApp", &argc, argv);
//!***********************************************************************
   int r, s, t;
  DrawPlot();
  Draw_dummy(0);
  Draw_dummy(1);
  ///++++++++++++++++++++++++++++++++++++++++
  theApp.Run();
  ///++++++++++++++++++++++++++++++++++++++++
  cout<< " plot finished "<<endl;
}///
