#include "TMCgen0.h"
#include <stdio.h>

////////////////////////////////////////////////////////////////////////////////
///     TMCgen0 class
/// This is class for axiliary exercises,  mainly integration with Monte Carlo

ClassImp(TMCgen0);

TMCgen0::TMCgen0():
  TMCgen()
{
  /// This constructor is for ROOT streamers ONLY
  cout<< "----> TMCgen0 Default Constructor (for ROOT only) "<<endl;
}

///_____________________________________________________________
TMCgen0::TMCgen0(const char* Name):
  TMCgen(Name)
{
//! all defaults defined here can be changed by the user
//! before calling TMCgen::Initialize
  m_pi = 3.14159;
  cout<< "----> TMCgen0::TMCgen0 USER Constructor "<<endl;
}///

///______________________________________________________________________________________
TMCgen0::~TMCgen0()
{
  //!Explicit destructor
  cout<< "----> TMCgen0::TMCgen0 !!!! DESTRUCTOR !!!! "<<endl;
}///destructor

///______________________________________________________________________________________
void TMCgen0::Initialize(TRandom *RNgen, ofstream *OutFile, TH1D* h_NORMA)
{
  cout<< "----> TMCgen0::Initialize, Entering "<<endl;
  ///	      SETTING UP RANDOM NUMBER GENERATOR
  TMCgen::Initialize(  RNgen, OutFile, h_NORMA);
  //if(f_IsInitialized == 0)
  //{
  ///         SETTING UP FOAM
  int nDim=2; // No. of dimensions for MC generator
  f_FoamI   = new TFoam("FoamI"); // new instance of MC generator FOAM
  f_FoamI->SetkDim(nDim);       //obligatory!
  f_FoamI->Initialize( f_RNgen, this);     // Initialize FOAM
  //screen output
  BXOPE(*f_Out);
  BXTXT(*f_Out,"========================================");
  BXTXT(*f_Out,"======   TMCgen0::Initialize     ======");
  BXTXT(*f_Out,"========================================");
  BXTXT(*f_Out,"========================================");
  f_IsInitialized = 1;  /// re-initilalization inhibited
  //}else{
    cout<< "----> TMCgen0::Initialize, already initialized "<<endl;
  //}
}/// Initialize

///______________________________________________________________________________________
void TMCgen0::Generate()
{
  f_NevGen++;
  f_FoamI->MakeEvent();         // generate x and y
  m_WT   = f_FoamI->GetMCwt();  // get weight
}//! Generate

///______________________________________________________________________________________
void TMCgen0::Finalize()
{
  TMCgen::Finalize();
  ///   Finalize MC  run, final printouts, cleaning etc.
  BXOPE(*f_Out);
  BXTXT(*f_Out,"****************************************");
  BXTXT(*f_Out,"******     TMCgen0::Finalize     ******");
  BXTXT(*f_Out,"****************************************");
  ///------------------------
  Double_t MCresult, MCerror;
  f_FoamI->GetIntegMC( MCresult, MCerror);  //! get MC integral, should be one
  cout << "**************************************************************"<<endl;
  cout << "**************** Foam::Finalize  ************************"<<endl;
  cout << "Directly from FOAM: MCresult= " << MCresult << " +- "<<MCerror <<endl;
  cout << "**************************************************************"<<endl;
  ///------------------------
}//!Finalize

///________________________________________________________________________
double TMCgen0::Density(int nDim, double *Xarg){
  double dist;
  m_x = Xarg[0];
  m_y = Xarg[1];
  double r = 0.45;
//   double z2 = sqr(m_x-0.5) + sqr(m_y-0.5);
//   if (z2 <= sqr(r))
//    dist = sqrt(r*r - z2)/(2./3 *3.14159*r*r*r);
//   return dist;
  double GamSq= sqr(0.100e0);
  double Dist= 0;
  Dist +=exp(-(sqr(m_x-1./3) +sqr(m_y-1./3))/GamSq)/GamSq/m_pi;
  Dist +=exp(-(sqr(m_x-2./3) +sqr(m_y-2./3))/GamSq)/GamSq/m_pi;
  return 0.5*Dist;


}//! Density
//===========================================================================


