#include "TRobol0.h"

////////////////////////////////////////////////////////////////////////////////
/// MC generator class for Two Parton exercises

ClassImp(TRobol0);

TRobol0::TRobol0():
  TRobol()
{
  /// This constructor is for ROOT streamers ONLY
  cout<< "@@@@> TRobol0 DEFAULT Constructor (for ROOT only) "<<endl;
}

///_____________________________________________________________
TRobol0::TRobol0(const char* Name):
  TRobol(Name)
{
//! Constructor to be used by the user!!!
//! Its important role is to define ALL DEFAULTS.
//! to changed by the user before calling TMCgen::Initialize
  cout<< "@@@@> TRobol0::TRobol0 USER Constructor "<<endl;
  m_xmin    = 0.01;        //! x range
  m_xmax    = 0.99;        //! x range

}///

///______________________________________________________________________________________
TRobol0::~TRobol0()
{
  //!Explicit destructor
  cout<< "@@@@> TRobol0::TRobol0 !!!! DESTRUCTOR !!!! "<<endl;
}///destructor

//______________________________________________________________________________
void TRobol0::Initialize(
        ofstream *OutFile, /// Central log-file for messages
        TFile *GenFile,    /// ROOT disk file for CRNG and MC gen.
        TFile *HstFile)    /// ROOT disk file for histograms
{
  cout<< "****> TRobol0::Initialize starts"<<endl;
  TRobol::Initialize(OutFile,GenFile,HstFile);
  ///
  /// Book histograms or read them from the disk
  Hbooker();
  cout<< "****> TRobol0::Initialize: finished"<<endl;
}///Initialize

///______________________________________________________________________________
void TRobol0::Hbooker()
{
  ///
  cout<< "****> TRobol0::Hbooker: histogram booking STARTS"<<endl;
  BXOPE(*f_Out);
  BXTXT(*f_Out,"========================================");
  BXTXT(*f_Out,"======    TRobol::Hbooker    ===========");
  BXTXT(*f_Out,"========================================");
  BXCLO(*f_Out);
  f_HstFile->cd();
/// Simple 1D histos
  h_wt    = TH1D_UP("h_wt",   " weight                  ",100,-0.5, 4.5);
  int    nbx=100;       /// linear scale
  double xmin= 0.001;   ///
  double xmax= 1.0001;
  h_x  = TH1D_UP("h_x"," dP/dx  ",nbx, 0.0, xmax);
  int Nblx=150;         /// log scale
  double xlmin= log10(xmin);
  double xlmax= log10(xmax);
  h_lx     =TH1D_UP("h_lx",    "log10(x) quark",Nblx,xlmin,xlmax);
/// Simple 2D histos
  int Nbx=50;
  int Nby=50;
 
  DistributionHisto = TH2D_UP("DistributionHisto"," x, y", Nbx, xmin, xmax, Nby, xmin,xmax);
  ///______________________________
  BXOPE(*f_Out);
  BXTXT(*f_Out,"========================================");
  BXTXT(*f_Out,"====== END of TRobol::Hbooker ==========");
  BXTXT(*f_Out,"========================================");
  BXCLO(*f_Out);
  cout<< "****> TRobol0::Hbooker: histogram booking FINISHED"<<endl;
}///Hbooker

///______________________________________________________________________________
void TRobol0::Production(double &iEvent)
{
  TRobol::Production(iEvent);  /// MC generation in base class
  ///
  double wt,xx, yy;
  ((TMCgen0*)f_MCgen)->GetMCevent(wt, xx, yy);
  h_x->Fill(xx,wt);
  //if ((wt > 1e3) || (fabs(wt) < 1e-8)) cout << "\t " << wt;
  double lg10=log(10.0);
  h_lx->Fill( log10(xx),wt/lg10/xx);
  double y1,y2;
double wt2D = wt/sqr(log(10));
  //! Result
  DistributionHisto->Fill( xx, yy, 1.0);
  
///
}///Production

